package de.fraunhofer.iem.dataprovider.kpi.metrics.repositoryDetails.service

import de.fraunhofer.iem.dataprovider.gitlab.service.OpenCodeGitlabApi
import de.fraunhofer.iem.dataprovider.kpi.dto.KPICreateDto
import de.fraunhofer.iem.dataprovider.kpi.metrics.repositoryDetails.dto.RepositoryDetailsKpisDto
import de.fraunhofer.iem.dataprovider.kpi.strategy.RawValueKPICalculationStrategy
import de.fraunhofer.iem.dataprovider.repository.dto.RepositoryDetailsDto
import de.fraunhofer.iem.dataprovider.tool.dto.CreateToolDto
import de.fraunhofer.iem.dataprovider.tool.entity.ToolEntity
import de.fraunhofer.iem.dataprovider.tool.service.ToolService
import org.springframework.stereotype.Service

@Service
class RepositoryDetailsService(private val openCodeGitlabApi: OpenCodeGitlabApi, private val toolService: ToolService) {

    suspend fun getRepositoryDetails(repoId: Long): RepositoryDetailsDto {
        return openCodeGitlabApi.getRepositoryDetails(repoId)
    }

    val toolEntity: ToolEntity = getOrCreateToolEntity()

    private final fun getOrCreateToolEntity(): ToolEntity {
        val createToolDto = CreateToolDto("Gitlab API")
        return toolService.findOrCreateTool(createToolDto)
    }

    /**
     * Creates a named map of RepositoryCreateDtos, based upon the provided repository details.
     * This method only returns raw KPIs.
     */
    fun calculateRepositoryDetailsKpis(repoDetailsDto: RepositoryDetailsDto): RepositoryDetailsKpisDto {

        val numberOfCommitsKPI = KPICreateDto(
            "Number of Commits",
            "Total number of commits on the default branch of the repository.",
            false,
            RawValueKPICalculationStrategy(repoDetailsDto.numberOfCommits)
        )
        val numberOfSignedCommitsKPI = KPICreateDto(
            "Number of Signed Commits",
            "Total number of signed and verified commits on the default branch of the repository.",
            false,
            RawValueKPICalculationStrategy(repoDetailsDto.numberOfSignedCommits)
        )
        val isDefaultBranchProtectedKPI = KPICreateDto(
            "Default Branch Protection",
            "Used to assess compliance with a standard development process. For this purpose, it is examined whether the standard development branch is protected against unintentional changes.",
            false,
            RawValueKPICalculationStrategy(if (repoDetailsDto.isDefaultBranchProtected) 100 else 0)
        )


        return RepositoryDetailsKpisDto(
            isDefaultBranchProtectedKPI = isDefaultBranchProtectedKPI,
            numberOfSignedCommitsKPI = numberOfSignedCommitsKPI,
            numberOfCommitsKpi = numberOfCommitsKPI
        )
    }
}
